package org.implab.gradle.mercurial;

import org.codehaus.groovy.runtime.MethodClosure;
import org.gradle.api.Plugin;
import org.gradle.api.Project;
import org.gradle.api.plugins.ExtraPropertiesExtension;
import org.gradle.api.tasks.TaskExecutionException;
import org.gradle.api.tasks.TaskProvider;
import org.implab.gradle.mercurial.tasks.HgBookmark;
import org.implab.gradle.mercurial.tasks.HgTag;
import org.implab.gradle.mercurial.tasks.HgTask;

public class MercurialPlugin implements Plugin<Project> {

    public final String MERCURIAL_EXTENSION_NAME = "mercurial";

    private MercurialExtension mercurialExtension;

    @Override
    public void apply(Project project) {

        mercurialExtension = new MercurialExtension(project);

        project.getExtensions().add(MERCURIAL_EXTENSION_NAME, mercurialExtension);

        ExtraPropertiesExtension extras = project.getExtensions().getExtraProperties();
        extras.set(HgTask.class.getSimpleName(), HgTask.class);
        extras.set("semver", new MethodClosure(SemVersion.class, "toSemVersion"));

        project.getTasks().register("printVersion", t -> {
            t.doLast(t2 -> {
                t2.getLogger().quiet("workspace: {}", mercurialExtension.getWorkspaceInfo());
                t2.getLogger().quiet("version: {}", project.getVersion());
            });
        });

        TaskProvider<HgBookmark> hgBookmarkTask = project.getTasks().register("hgBookmark", HgBookmark.class, t -> {
            t.getInactive().set(true);
            t.getForce().set(true);
        });

        TaskProvider<HgTag> hgTagTask = project.getTasks().register("hgTag", HgTag.class, t -> {
            t.mustRunAfter(hgBookmarkTask);
        });

        project.getTasks().register("release", t -> {
            t.finalizedBy(hgBookmarkTask, hgTagTask);
            t.doLast(self -> {
                try {
                    if (mercurialExtension.checkIsDirty())
                        throw new Exception("The workspace is dirty, you need to commit changes first");

                    SemVersion version = (SemVersion) project.getVersion();
                    SemVersion nextVersion = version.withNoSuffix();

                    self.getLogger().quiet("{} {} -> {}", mercurialExtension.getReleaseBookmark(), version, nextVersion);

                    hgBookmarkTask.get().getBookmarks().add(mercurialExtension.getReleaseBookmark());
                    hgTagTask.get().getTags().add("v" + nextVersion.toString());
                } catch (Exception e) {
                    throw new TaskExecutionException(self, e);
                }
            });
        });

        project.getTasks().register("staging", t -> {
            t.finalizedBy(hgBookmarkTask, hgTagTask);
            t.doLast(self -> {
                try {
                    if (mercurialExtension.checkIsDirty())
                        throw new Exception("The workspace is dirty, you need to commit changes first");

                    SemVersion version = (SemVersion) project.getVersion();
                    SemVersion nextVersion = version.withNoSuffix().withPreRelease("rc");

                    self.getLogger().quiet("{} {} -> {}", mercurialExtension.getStagingBookmark(), version, nextVersion);

                    hgBookmarkTask.get().getBookmarks().add(mercurialExtension.getStagingBookmark());
                    hgTagTask.get().getTags().add("v" + nextVersion.toString());
                } catch (Exception e) {
                    throw new TaskExecutionException(self, e);
                }
            });
        });
    }
}
